@@ -1,16 +1,13 @@
 import uuid
 from pathlib import Path
+from pydantic_ai.models.openai import OpenAIModelSettings, OpenAIResponsesModelSettings
 from typing import Dict, Optional
 
 from pydantic_ai import Agent
 from pydantic_ai.settings import ModelSettings
 from pydantic_ai.usage import UsageLimits
 
-from code_puppy.agents import get_current_agent_config
-from code_puppy.message_history_processor import (
-    get_model_context_length,
-    message_history_accumulator,
-)
+from code_puppy.message_history_processor import message_history_accumulator
 from code_puppy.messaging.message_queue import (
     emit_error,
     emit_info,
@@ -24,17 +21,17 @@ from code_puppy.tools.common import console
 
 def load_puppy_rules():
     global PUPPY_RULES
-    
+
     # Check for all 4 combinations of the rules file
     possible_paths = ["AGENTS.md", "AGENT.md", "agents.md", "agent.md"]
-    
+
     for path_str in possible_paths:
         puppy_rules_path = Path(path_str)
         if puppy_rules_path.exists():
             with open(puppy_rules_path, "r") as f:
                 puppy_rules = f.read()
                 return puppy_rules
-    
+
     # If none of the files exist, return None
     return None
 
@@ -48,7 +45,7 @@ _code_generation_agent = None
 def _load_mcp_servers(extra_headers: Optional[Dict[str, str]] = None):
     """Load MCP servers using the new manager while maintaining backward compatibility."""
     from code_puppy.config import get_value, load_mcp_server_configs
-    from code_puppy.mcp import ServerConfig, get_mcp_manager
+    from code_puppy.mcp_ import ServerConfig, get_mcp_manager
 
     # Check if MCP servers are disabled
     mcp_disabled = get_value("disable_mcp_servers")
@@ -113,7 +110,7 @@ def _load_mcp_servers(extra_headers: Optional[Dict[str, str]] = None):
 
 def reload_mcp_servers():
     """Reload MCP servers without restarting the agent."""
-    from code_puppy.mcp import get_mcp_manager
+    from code_puppy.mcp_ import get_mcp_manager
 
     manager = get_mcp_manager()
     # Reload configurations
@@ -134,7 +131,16 @@ def reload_code_generation_agent(message_group: str | None):
     clear_model_cache()
     clear_agent_cache()
 
-    model_name = get_model_name()
+    # Check if current agent has a pinned model
+    from code_puppy.agents import get_current_agent_config
+
+    agent_config = get_current_agent_config()
+    agent_model_name = None
+    if hasattr(agent_config, "get_model_name"):
+        agent_model_name = agent_config.get_model_name()
+
+    # Use agent-specific model if pinned, otherwise use global model
+    model_name = agent_model_name if agent_model_name else get_model_name()
     emit_info(
         f"[bold cyan]Loading Model: {model_name}[/bold cyan]",
         message_group=message_group,
@@ -158,11 +164,21 @@ def reload_code_generation_agent(message_group: str | None):
 
     # Configure model settings with max_tokens if set
     model_settings_dict = {"seed": 42}
-    output_tokens = max(2048, min(int(0.05 * get_model_context_length()) - 1024, 16384))
+    # Get current agent to use its method
+    from code_puppy.agents import get_current_agent_config
+    current_agent = get_current_agent_config()
+    output_tokens = max(2048, min(int(0.05 * current_agent.get_model_context_length()) - 1024, 16384))
     console.print(f"Max output tokens per message: {output_tokens}")
     model_settings_dict["max_tokens"] = output_tokens
 
+
     model_settings = ModelSettings(**model_settings_dict)
+    if "gpt-5" in model_name:
+        model_settings_dict["openai_reasoning_effort"] = "off"
+        model_settings_dict["extra_body"] = {
+            "verbosity": "low"
+        }
+        model_settings = OpenAIModelSettings(**model_settings_dict)
     agent = Agent(
         model=model,
         instructions=instructions,
@@ -193,7 +209,20 @@ def get_code_generation_agent(force_reload=False, message_group: str | None = No
         message_group = str(uuid.uuid4())
     from code_puppy.config import get_model_name
 
-    model_name = get_model_name()
+    # Get the global model name
+    global_model_name = get_model_name()
+
+    # Check if current agent has a pinned model
+    from code_puppy.agents import get_current_agent_config
+
+    agent_config = get_current_agent_config()
+    agent_model_name = None
+    if hasattr(agent_config, "get_model_name"):
+        agent_model_name = agent_config.get_model_name()
+
+    # Use agent-specific model if pinned, otherwise use global model
+    model_name = agent_model_name if agent_model_name else global_model_name
+
     if _code_generation_agent is None or _LAST_MODEL_NAME != model_name or force_reload:
         return reload_code_generation_agent(message_group)
     return _code_generation_agent
@@ -206,4 +235,5 @@ def get_custom_usage_limits():
     Default pydantic-ai limit is 50, this increases it to the configured value (default 100).
     """
     from code_puppy.config import get_message_limit
+
     return UsageLimits(request_limit=get_message_limit())
