@@ -1,9 +1,6 @@
 import os
 
-from code_puppy.command_line.model_picker_completion import (
-    load_model_names,
-    update_model_in_input,
-)
+from code_puppy.command_line.model_picker_completion import update_model_in_input
 from code_puppy.command_line.motd import print_motd
 from code_puppy.command_line.utils import make_directory_table
 from code_puppy.config import get_config_keys
@@ -42,6 +39,10 @@ def get_commands_help():
     help_lines.append(
         Text("/model, /m", style="cyan") + Text(" <model>   Set active model")
     )
+    help_lines.append(
+        Text("/pin_model", style="cyan")
+        + Text(" <agent> <model>  Pin a specific model to an agent")
+    )
     help_lines.append(
         Text("/mcp", style="cyan")
         + Text("                  Manage MCP servers (list, start, stop, status, etc.)")
@@ -56,7 +57,9 @@ def get_commands_help():
     )
     help_lines.append(
         Text("/compact", style="cyan")
-        + Text("              Summarize and compact current chat history")
+        + Text(
+            "              Summarize and compact current chat history (uses compaction_strategy config)"
+        )
     )
     help_lines.append(
         Text("/dump_context", style="cyan")
@@ -76,6 +79,12 @@ def get_commands_help():
         Text("/tools", style="cyan")
         + Text("                Show available tools and capabilities")
     )
+    help_lines.append(
+        Text("/truncate", style="cyan")
+        + Text(
+            " <N>              Truncate message history to N most recent messages (keeping system message)"
+        )
+    )
     help_lines.append(
         Text("/<unknown>", style="cyan")
         + Text("            Show unknown command warning")
@@ -133,18 +142,18 @@ def handle_command(command: str):
 
             before_tokens = sum(estimate_tokens_for_message(m) for m in history)
             compaction_strategy = get_compaction_strategy()
+            protected_tokens = get_protected_token_count()
             emit_info(
                 f"🤔 Compacting {len(history)} messages using {compaction_strategy} strategy... (~{before_tokens} tokens)"
             )
 
             if compaction_strategy == "truncation":
-                protected_tokens = get_protected_token_count()
                 compacted = truncation(history, protected_tokens)
                 summarized_messages = []  # No summarization in truncation mode
             else:
                 # Default to summarization
                 compacted, summarized_messages = summarize_messages(
-                    history, with_protection=False
+                    history, with_protection=True
                 )
 
             if not compacted:
@@ -368,6 +377,9 @@ def handle_command(command: str):
             # Convert /model to /m for internal processing
             model_command = command.replace("/model", "/m", 1)
 
+        # If no model matched, show available models
+        from code_puppy.command_line.model_picker_completion import load_model_names
+
         new_input = update_model_in_input(model_command)
         if new_input is not None:
             from code_puppy.agents.runtime_manager import get_runtime_agent_manager
@@ -379,7 +391,6 @@ def handle_command(command: str):
             manager.reload_agent()
             emit_success(f"Active model set and loaded: {model}")
             return True
-        # If no model matched, show available models
         model_names = load_model_names()
         emit_warning("Usage: /model <model-name> or /m <model-name>")
         emit_warning(f"Available models: {', '.join(model_names)}")
@@ -398,6 +409,115 @@ def handle_command(command: str):
         emit_info(help_text, message_group_id=group_id)
         return True
 
+    if command.startswith("/pin_model"):
+        # Handle agent model pinning
+        import json
+
+        from code_puppy.agents.json_agent import discover_json_agents
+        from code_puppy.command_line.model_picker_completion import load_model_names
+
+        tokens = command.split()
+
+        if len(tokens) != 3:
+            emit_warning("Usage: /pin_model <agent-name> <model-name>")
+
+            # Show available models and agents
+            available_models = load_model_names()
+            json_agents = discover_json_agents()
+
+            # Get built-in agents
+            from code_puppy.agents.agent_manager import get_agent_descriptions
+            builtin_agents = get_agent_descriptions()
+
+            emit_info("Available models:")
+            for model in available_models:
+                emit_info(f"  [cyan]{model}[/cyan]")
+
+            if builtin_agents:
+                emit_info("\nAvailable built-in agents:")
+                for agent_name, description in builtin_agents.items():
+                    emit_info(f"  [cyan]{agent_name}[/cyan] - {description}")
+
+            if json_agents:
+                emit_info("\nAvailable JSON agents:")
+                for agent_name, agent_path in json_agents.items():
+                    emit_info(f"  [cyan]{agent_name}[/cyan] ({agent_path})")
+            return True
+
+        agent_name = tokens[1].lower()
+        model_name = tokens[2]
+
+        # Check if model exists
+        available_models = load_model_names()
+        if model_name not in available_models:
+            emit_error(f"Model '{model_name}' not found")
+            emit_warning(f"Available models: {', '.join(available_models)}")
+            return True
+
+        # Check if this is a JSON agent or a built-in Python agent
+        json_agents = discover_json_agents()
+
+        # Get list of available built-in agents
+        from code_puppy.agents.agent_manager import get_agent_descriptions
+        builtin_agents = get_agent_descriptions()
+
+        is_json_agent = agent_name in json_agents
+        is_builtin_agent = agent_name in builtin_agents
+
+        if not is_json_agent and not is_builtin_agent:
+            emit_error(f"Agent '{agent_name}' not found")
+
+            # Show available agents
+            if builtin_agents:
+                emit_info("Available built-in agents:")
+                for name, desc in builtin_agents.items():
+                    emit_info(f"  [cyan]{name}[/cyan] - {desc}")
+
+            if json_agents:
+                emit_info("\nAvailable JSON agents:")
+                for name, path in json_agents.items():
+                    emit_info(f"  [cyan]{name}[/cyan] ({path})")
+            return True
+
+        # Handle different agent types
+        try:
+            if is_json_agent:
+                # Handle JSON agent - modify the JSON file
+                agent_file_path = json_agents[agent_name]
+
+                with open(agent_file_path, "r", encoding="utf-8") as f:
+                    agent_config = json.load(f)
+
+                # Set the model
+                agent_config["model"] = model_name
+
+                # Save the updated configuration
+                with open(agent_file_path, "w", encoding="utf-8") as f:
+                    json.dump(agent_config, f, indent=2, ensure_ascii=False)
+
+            else:
+                # Handle built-in Python agent - store in config
+                from code_puppy.config import set_agent_pinned_model
+                set_agent_pinned_model(agent_name, model_name)
+
+            emit_success(f"Model '{model_name}' pinned to agent '{agent_name}'")
+
+            # If this is the current agent, reload it to use the new model
+            from code_puppy.agents import get_current_agent_config
+            from code_puppy.agents.runtime_manager import get_runtime_agent_manager
+
+            current_agent = get_current_agent_config()
+            if current_agent.name == agent_name:
+                manager = get_runtime_agent_manager()
+                manager.reload_agent()
+                emit_info(f"Active agent reloaded with pinned model '{model_name}'")
+
+            return True
+
+        except Exception as e:
+            emit_error(f"Failed to pin model to agent '{agent_name}': {e}")
+            return True
+
     if command.startswith("/generate-pr-description"):
         # Parse directory argument (e.g., /generate-pr-description @some/dir)
         tokens = command.split()
@@ -531,6 +651,47 @@ def handle_command(command: str):
             emit_error(f"Failed to load context: {e}")
             return True
 
+    if command.startswith("/truncate"):
+        tokens = command.split()
+        if len(tokens) != 2:
+            emit_error(
+                "Usage: /truncate <N> (where N is the number of messages to keep)"
+            )
+            return True
+
+        try:
+            n = int(tokens[1])
+            if n < 1:
+                emit_error("N must be a positive integer")
+                return True
+        except ValueError:
+            emit_error("N must be a valid integer")
+            return True
+
+        from code_puppy.state_management import get_message_history, set_message_history
+
+        history = get_message_history()
+        if not history:
+            emit_warning("No history to truncate yet. Ask me something first!")
+            return True
+
+        if len(history) <= n:
+            emit_info(
+                f"History already has {len(history)} messages, which is <= {n}. Nothing to truncate."
+            )
+            return True
+
+        # Always keep the first message (system message) and then keep the N-1 most recent messages
+        truncated_history = (
+            [history[0]] + history[-(n - 1) :] if n > 1 else [history[0]]
+        )
+
+        set_message_history(truncated_history)
+        emit_success(
+            f"Truncated message history from {len(history)} to {len(truncated_history)} messages (keeping system message and {n - 1} most recent)"
+        )
+        return True
+
     if command in ("/exit", "/quit"):
         emit_success("Goodbye!")
         # Signal to the main app that we want to exit
