@@ -14,6 +14,12 @@ AGENTS_DIR = os.path.join(CONFIG_DIR, "agents")
 DEFAULT_SECTION = "puppy"
 REQUIRED_KEYS = ["puppy_name", "owner_name"]
 
+# Cache containers for model validation and defaults
+_model_validation_cache = {}
+_default_model_cache = None
+_default_vision_model_cache = None
+_default_vqa_model_cache = None
+
 
 def ensure_config_exists():
     """
@@ -106,7 +112,15 @@ def get_config_keys():
     Returns the list of all config keys currently in puppy.cfg,
     plus certain preset expected keys (e.g. "yolo_mode", "model", "compaction_strategy", "message_limit", "allow_recursion").
     """
-    default_keys = ["yolo_mode", "model", "compaction_strategy", "message_limit", "allow_recursion"]
+    default_keys = [
+        "yolo_mode",
+        "model",
+        "compaction_strategy",
+        "protected_token_count",
+        "compaction_threshold",
+        "message_limit",
+        "allow_recursion",
+    ]
     config = configparser.ConfigParser()
     config.read(CONFIG_FILE)
     keys = set(config[DEFAULT_SECTION].keys()) if DEFAULT_SECTION in config else set()
@@ -148,35 +162,115 @@ def load_mcp_server_configs():
         return {}
 
 
-# Cache for model validation to prevent hitting ModelFactory on every call
-_model_validation_cache = {}
-_default_model_cache = None
 
 
 def _default_model_from_models_json():
     """Attempt to load the first model name from models.json.
 
-    Falls back to the hard-coded default (``claude-4-0-sonnet``) if the file
+    Falls back to the hard-coded default (``gpt-5``) if the file
     cannot be read for any reason or is empty.
     """
     global _default_model_cache
 
-    # Return cached default if we have one
     if _default_model_cache is not None:
         return _default_model_cache
 
     try:
-        # Local import to avoid potential circular dependency on module import
         from code_puppy.model_factory import ModelFactory
 
         models_config = ModelFactory.load_config()
-        first_key = next(iter(models_config))  # Raises StopIteration if empty
-        _default_model_cache = first_key
-        return first_key
-    except Exception as e:
-        # Any problem (network, file missing, empty dict, etc.) => fall back
+        if models_config:
+            first_key = next(iter(models_config))
+            _default_model_cache = first_key
+            return first_key
         _default_model_cache = "gpt-5"
         return "gpt-5"
+    except Exception:
+        _default_model_cache = "gpt-5"
+        return "gpt-5"
+
+
+def _default_vision_model_from_models_json() -> str:
+    """Select a default vision-capable model from models.json with caching."""
+    global _default_vision_model_cache
+
+    if _default_vision_model_cache is not None:
+        return _default_vision_model_cache
+
+    try:
+        from code_puppy.model_factory import ModelFactory
+
+        models_config = ModelFactory.load_config()
+        if models_config:
+            # Prefer explicitly tagged vision models
+            for name, config in models_config.items():
+                if config.get("supports_vision"):
+                    _default_vision_model_cache = name
+                    return name
+
+            # Fallback heuristic: common multimodal models
+            preferred_candidates = (
+                "gpt-4.1",
+                "gpt-4.1-mini",
+                "gpt-4.1-nano",
+                "claude-4-0-sonnet",
+                "gemini-2.5-flash-preview-05-20",
+            )
+            for candidate in preferred_candidates:
+                if candidate in models_config:
+                    _default_vision_model_cache = candidate
+                    return candidate
+
+            # Last resort: use the general default model
+            _default_vision_model_cache = _default_model_from_models_json()
+            return _default_vision_model_cache
+
+        _default_vision_model_cache = "gpt-4.1"
+        return "gpt-4.1"
+    except Exception:
+        _default_vision_model_cache = "gpt-4.1"
+        return "gpt-4.1"
+
+
+def _default_vqa_model_from_models_json() -> str:
+    """Select a default VQA-capable model, preferring vision-ready options."""
+    global _default_vqa_model_cache
+
+    if _default_vqa_model_cache is not None:
+        return _default_vqa_model_cache
+
+    try:
+        from code_puppy.model_factory import ModelFactory
+
+        models_config = ModelFactory.load_config()
+        if models_config:
+            # Allow explicit VQA hints if present
+            for name, config in models_config.items():
+                if config.get("supports_vqa"):
+                    _default_vqa_model_cache = name
+                    return name
+
+            # Reuse multimodal heuristics before falling back to generic default
+            preferred_candidates = (
+                "gpt-4.1",
+                "gpt-4.1-mini",
+                "claude-4-0-sonnet",
+                "gemini-2.5-flash-preview-05-20",
+                "gpt-4.1-nano",
+            )
+            for candidate in preferred_candidates:
+                if candidate in models_config:
+                    _default_vqa_model_cache = candidate
+                    return candidate
+
+            _default_vqa_model_cache = _default_model_from_models_json()
+            return _default_vqa_model_cache
+
+        _default_vqa_model_cache = "gpt-4.1"
+        return "gpt-4.1"
+    except Exception:
+        _default_vqa_model_cache = "gpt-4.1"
+        return "gpt-4.1"
 
 
 def _validate_model_exists(model_name: str) -> bool:
@@ -190,8 +284,7 @@ def _validate_model_exists(model_name: str) -> bool:
     try:
         from code_puppy.model_factory import ModelFactory
 
-        models_config_path = os.path.join(CONFIG_DIR, "models.json")
-        models_config = ModelFactory.load_config(models_config_path)
+        models_config = ModelFactory.load_config()
         exists = model_name in models_config
 
         # Cache the result
@@ -205,9 +298,11 @@ def _validate_model_exists(model_name: str) -> bool:
 
 def clear_model_cache():
     """Clear the model validation cache. Call this when models.json changes."""
-    global _model_validation_cache, _default_model_cache
+    global _model_validation_cache, _default_model_cache, _default_vision_model_cache, _default_vqa_model_cache
     _model_validation_cache.clear()
     _default_model_cache = None
+    _default_vision_model_cache = None
+    _default_vqa_model_cache = None
 
 
 def get_model_name():
@@ -245,6 +340,20 @@ def set_model_name(model: str):
     clear_model_cache()
 
 
+def get_vqa_model_name() -> str:
+    """Return the configured VQA model, falling back to an inferred default."""
+    stored_model = get_value("vqa_model_name")
+    if stored_model and _validate_model_exists(stored_model):
+        return stored_model
+    return _default_vqa_model_from_models_json()
+
+
+def set_vqa_model_name(model: str):
+    """Persist the configured VQA model name and refresh caches."""
+    set_config_value("vqa_model_name", model or "")
+    clear_model_cache()
+
+
 def get_puppy_token():
     """Returns the puppy_token from config, or None if not set."""
     return get_value("puppy_token")
@@ -480,3 +589,36 @@ def save_command_to_history(command: str):
             f"❌ An unexpected error occurred while saving command history: {str(e)}"
         )
         direct_console.print(f"[bold red]{error_msg}[/bold red]")
+
+
+def get_agent_pinned_model(agent_name: str) -> str:
+    """Get the pinned model for a specific agent.
+
+    Args:
+        agent_name: Name of the agent to get the pinned model for.
+
+    Returns:
+        Pinned model name, or None if no model is pinned for this agent.
+    """
+    return get_value(f"agent_model_{agent_name}")
+
+
+def set_agent_pinned_model(agent_name: str, model_name: str):
+    """Set the pinned model for a specific agent.
+
+    Args:
+        agent_name: Name of the agent to pin the model for.
+        model_name: Model name to pin to this agent.
+    """
+    set_config_value(f"agent_model_{agent_name}", model_name)
+
+
+def clear_agent_pinned_model(agent_name: str):
+    """Clear the pinned model for a specific agent.
+
+    Args:
+        agent_name: Name of the agent to clear the pinned model for.
+    """
+    # We can't easily delete keys from configparser, so set to empty string
+    # which will be treated as None by get_agent_pinned_model
+    set_config_value(f"agent_model_{agent_name}", "")
