@@ -6,15 +6,15 @@ from typing import Any, Dict
 
 import httpx
 from anthropic import AsyncAnthropic
-from openai import AsyncAzureOpenAI  # For Azure OpenAI client
+from openai import AsyncAzureOpenAI
 from pydantic_ai.models.anthropic import AnthropicModel
-from pydantic_ai.models.fallback import infer_model
 from pydantic_ai.models.gemini import GeminiModel
 from pydantic_ai.models.openai import OpenAIChatModel
 from pydantic_ai.providers.anthropic import AnthropicProvider
 from pydantic_ai.providers.google_gla import GoogleGLAProvider
 from pydantic_ai.providers.openai import OpenAIProvider
 from pydantic_ai.providers.cerebras import CerebrasProvider
+from pydantic_ai.providers.openrouter import OpenRouterProvider
 
 from . import callbacks
 from .config import EXTRA_MODELS_FILE
@@ -87,17 +87,28 @@ class ModelFactory:
         else:
             from code_puppy.config import MODELS_FILE
 
-            if not pathlib.Path(MODELS_FILE).exists():
-                with open(pathlib.Path(__file__).parent / "models.json", "r") as src:
-                    with open(pathlib.Path(MODELS_FILE), "w") as target:
-                        target.write(src.read())
+            with open(pathlib.Path(__file__).parent / "models.json", "r") as src:
+                with open(pathlib.Path(MODELS_FILE), "w") as target:
+                    target.write(src.read())
 
             with open(MODELS_FILE, "r") as f:
                 config = json.load(f)
+
         if pathlib.Path(EXTRA_MODELS_FILE).exists():
-            with open(EXTRA_MODELS_FILE, "r") as f:
-                extra_config = json.load(f)
-                config.update(extra_config)
+            try:
+                with open(EXTRA_MODELS_FILE, "r") as f:
+                    extra_config = json.load(f)
+                    config.update(extra_config)
+            except json.JSONDecodeError as e:
+                logging.getLogger(__name__).warning(
+                    f"Failed to load extra models config from {EXTRA_MODELS_FILE}: Invalid JSON - {e}\n"
+                    f"Please check your extra_models.json file for syntax errors."
+                )
+            except Exception as e:
+                logging.getLogger(__name__).warning(
+                    f"Failed to load extra models config from {EXTRA_MODELS_FILE}: {e}\n"
+                    f"The extra models configuration will be ignored."
+                )
         return config
 
     @staticmethod
@@ -248,25 +259,55 @@ class ModelFactory:
             model = OpenAIChatModel(model_name=model_config["name"], provider=provider)
             setattr(model, "provider", provider)
             return model
-        
+
+        elif model_type == "openrouter":
+            # Get API key from config, which can be an environment variable reference or raw value
+            api_key_config = model_config.get("api_key")
+            api_key = None
+
+            if api_key_config:
+                if api_key_config.startswith("$"):
+                    # It's an environment variable reference
+                    env_var_name = api_key_config[1:]  # Remove the $ prefix
+                    api_key = os.environ.get(env_var_name)
+                    if api_key is None:
+                        raise ValueError(
+                            f"OpenRouter API key environment variable '{env_var_name}' not found or is empty. "
+                            f"Please set the environment variable: export {env_var_name}=your_value"
+                        )
+                else:
+                    # It's a raw API key value
+                    api_key = api_key_config
+            else:
+                # No API key in config, try to get it from the default environment variable
+                api_key = os.environ.get("OPENROUTER_API_KEY")
+
+            provider = OpenRouterProvider(api_key=api_key)
+
+            model = OpenAIChatModel(model_name=model_config["name"], provider=provider)
+            setattr(model, "provider", provider)
+            return model
+
         elif model_type == "round_robin":
             # Get the list of model names to use in the round-robin
             model_names = model_config.get("models")
             if not model_names or not isinstance(model_names, list):
-                raise ValueError(f"Round-robin model '{model_name}' requires a 'models' list in its configuration.")
-            
+                raise ValueError(
+                    f"Round-robin model '{model_name}' requires a 'models' list in its configuration."
+                )
+
             # Get the rotate_every parameter (default: 1)
             rotate_every = model_config.get("rotate_every", 1)
-            
+
             # Resolve each model name to an actual model instance
             models = []
             for name in model_names:
                 # Recursively get each model using the factory
                 model = ModelFactory.get_model(name, config)
                 models.append(model)
-            
+
             # Create and return the round-robin model
             return RoundRobinModel(*models, rotate_every=rotate_every)
-        
+
         else:
             raise ValueError(f"Unsupported model type: {model_type}")
