@@ -12,7 +12,6 @@ from pydantic_ai import RunContext
 from rich.markdown import Markdown
 from rich.text import Text
 
-from code_puppy.callbacks import on_run_shell_command
 from code_puppy.messaging import (
     emit_divider,
     emit_error,
@@ -20,9 +19,21 @@ from code_puppy.messaging import (
     emit_system_message,
     emit_warning,
 )
-from code_puppy.state_management import is_tui_mode
+from code_puppy.tui_state import is_tui_mode
 from code_puppy.tools.common import generate_group_id
 
+# Maximum line length for shell command output to prevent massive token usage
+# This helps avoid exceeding model context limits when commands produce very long lines
+MAX_LINE_LENGTH = 256
+
+
+def _truncate_line(line: str) -> str:
+    """Truncate a line to MAX_LINE_LENGTH if it exceeds the limit."""
+    if len(line) > MAX_LINE_LENGTH:
+        return line[:MAX_LINE_LENGTH] + "... [truncated]"
+    return line
+
+
 _AWAITING_USER_INPUT = False
 
 _CONFIRMATION_LOCK = threading.Lock()
@@ -189,6 +200,8 @@ def run_shell_command_streaming(
             for line in iter(process.stdout.readline, ""):
                 if line:
                     line = line.rstrip("\n\r")
+                    # Limit line length to prevent massive token usage
+                    line = _truncate_line(line)
                     stdout_lines.append(line)
                     emit_system_message(line, message_group=group_id)
                     last_output_time[0] = time.time()
@@ -200,6 +213,8 @@ def run_shell_command_streaming(
             for line in iter(process.stderr.readline, ""):
                 if line:
                     line = line.rstrip("\n\r")
+                    # Limit line length to prevent massive token usage
+                    line = _truncate_line(line)
                     stderr_lines.append(line)
                     emit_system_message(line, message_group=group_id)
                     last_output_time[0] = time.time()
@@ -253,8 +268,8 @@ def run_shell_command_streaming(
             **{
                 "success": False,
                 "command": command,
-                "stdout": "\n".join(stdout_lines[-1000:]),
-                "stderr": "\n".join(stderr_lines[-1000:]),
+                "stdout": "\n".join(stdout_lines[-256:]),
+                "stderr": "\n".join(stderr_lines[-256:]),
                 "exit_code": -9,
                 "execution_time": execution_time,
                 "timeout": True,
@@ -316,23 +331,31 @@ def run_shell_command_streaming(
             )
             emit_info(f"Took {execution_time:.2f}s", message_group=group_id)
             time.sleep(1)
+            # Apply line length limits to stdout/stderr before returning
+            truncated_stdout = [_truncate_line(line) for line in stdout_lines[-256:]]
+            truncated_stderr = [_truncate_line(line) for line in stderr_lines[-256:]]
+
             return ShellCommandOutput(
                 success=False,
                 command=command,
                 error="""The process didn't exit cleanly! If the user_interrupted flag is true,
                 please stop all execution and ask the user for clarification!""",
-                stdout="\n".join(stdout_lines[-1000:]),
-                stderr="\n".join(stderr_lines[-1000:]),
+                stdout="\n".join(truncated_stdout),
+                stderr="\n".join(truncated_stderr),
                 exit_code=exit_code,
                 execution_time=execution_time,
                 timeout=False,
                 user_interrupted=process.pid in _USER_KILLED_PROCESSES,
             )
+        # Apply line length limits to stdout/stderr before returning
+        truncated_stdout = [_truncate_line(line) for line in stdout_lines[-256:]]
+        truncated_stderr = [_truncate_line(line) for line in stderr_lines[-256:]]
+
         return ShellCommandOutput(
             success=exit_code == 0,
             command=command,
-            stdout="\n".join(stdout_lines[-1000:]),
-            stderr="\n".join(stderr_lines[-1000:]),
+            stdout="\n".join(truncated_stdout),
+            stderr="\n".join(truncated_stderr),
             exit_code=exit_code,
             execution_time=execution_time,
             timeout=False,
@@ -454,12 +477,28 @@ def run_shell_command(
             stdout = None
         if "stderr" not in locals():
             stderr = None
+
+        # Apply line length limits to stdout/stderr if they exist
+        truncated_stdout = None
+        if stdout:
+            stdout_lines = stdout.split("\n")
+            truncated_stdout = "\n".join(
+                [_truncate_line(line) for line in stdout_lines[-256:]]
+            )
+
+        truncated_stderr = None
+        if stderr:
+            stderr_lines = stderr.split("\n")
+            truncated_stderr = "\n".join(
+                [_truncate_line(line) for line in stderr_lines[-256:]]
+            )
+
         return ShellCommandOutput(
             success=False,
             command=command,
             error=f"Error executing command {str(e)}",
-            stdout="\n".join(stdout[-1000:]) if stdout else None,
-            stderr="\n".join(stderr[-1000:]) if stderr else None,
+            stdout=truncated_stdout,
+            stderr=truncated_stderr,
             exit_code=-1,
             timeout=False,
         )
@@ -494,108 +533,10 @@ def share_your_reasoning(
     return ReasoningOutput(**{"success": True})
 
 
-def register_command_runner_tools(agent):
-    @agent.tool
-    def agent_run_shell_command(
-        context: RunContext, command: str = "", cwd: str = None, timeout: int = 60
-    ) -> ShellCommandOutput:
-        """Execute a shell command with comprehensive monitoring and safety features.
-
-        This tool provides robust shell command execution with streaming output,
-        timeout handling, user confirmation (when not in yolo mode), and proper
-        process lifecycle management. Commands are executed in a controlled
-        environment with cross-platform process group handling.
-
-        Args:
-            context (RunContext): The PydanticAI runtime context for the agent.
-            command (str): The shell command to execute. Cannot be empty or whitespace-only.
-            cwd (str, optional): Working directory for command execution. If None,
-                uses the current working directory. Defaults to None.
-            timeout (int, optional): Inactivity timeout in seconds. If no output is
-                produced for this duration, the process will be terminated.
-                Defaults to 60 seconds.
-
-        Returns:
-            ShellCommandOutput: A structured response containing:
-                - success (bool): True if command executed successfully (exit code 0)
-                - command (str | None): The executed command string
-                - error (str | None): Error message if execution failed
-                - stdout (str | None): Standard output from the command (last 1000 lines)
-                - stderr (str | None): Standard error from the command (last 1000 lines)
-                - exit_code (int | None): Process exit code
-                - execution_time (float | None): Total execution time in seconds
-                - timeout (bool | None): True if command was terminated due to timeout
-                - user_interrupted (bool | None): True if user killed the process
-
-        Note:
-            - In interactive mode (not yolo), user confirmation is required before execution
-            - Commands have an absolute timeout of 270 seconds regardless of activity
-            - Process groups are properly managed for clean termination
-            - Output is streamed in real-time and displayed to the user
-            - Large output is truncated to the last 1000 lines for memory efficiency
-
-        Examples:
-            >>> result = agent_run_shell_command(ctx, "ls -la", cwd="/tmp", timeout=30)
-            >>> if result.success:
-            ...     print(f"Command completed in {result.execution_time:.2f}s")
-            ...     print(result.stdout)
-
-        Warning:
-            This tool can execute arbitrary shell commands. Exercise caution when
-            running untrusted commands, especially those that modify system state.
-        """
-        result = run_shell_command(context, command, cwd, timeout)
-        on_run_shell_command(result)
-
-    @agent.tool
-    def agent_share_your_reasoning(
-        context: RunContext, reasoning: str = "", next_steps: str | None = None
-    ) -> ReasoningOutput:
-        """Share the agent's current reasoning and planned next steps with the user.
-
-        This tool provides transparency into the agent's decision-making process
-        by displaying the current reasoning and upcoming actions in a formatted,
-        user-friendly manner. It's essential for building trust and understanding
-        between the agent and user.
-
-        Args:
-            context (RunContext): The PydanticAI runtime context for the agent.
-            reasoning (str): The agent's current thought process, analysis, or
-                reasoning for the current situation. This should be clear,
-                comprehensive, and explain the 'why' behind decisions.
-            next_steps (str | None, optional): Planned upcoming actions or steps
-                the agent intends to take. Can be None if no specific next steps
-                are determined. Defaults to None.
-
-        Returns:
-            ReasoningOutput: A simple response object containing:
-                - success (bool): Always True, indicating the reasoning was shared
-
-        Note:
-            - Reasoning is displayed with Markdown formatting for better readability
-            - Next steps are only shown if provided and non-empty
-            - Output is visually separated with dividers in TUI mode
-            - This tool should be called before major actions to explain intent
-
-        Examples:
-            >>> reasoning = "I need to analyze the codebase structure before making changes"
-            >>> next_steps = "First, I'll list the directory contents, then read key files"
-            >>> result = agent_share_your_reasoning(ctx, reasoning, next_steps)
-
-        Best Practice:
-            Use this tool frequently to maintain transparency. Call it:
-            - Before starting complex operations
-            - When changing strategy or approach
-            - To explain why certain decisions are being made
-            - When encountering unexpected situations
-        """
-        return share_your_reasoning(context, reasoning, next_steps)
-
-
 def register_agent_run_shell_command(agent):
     """Register only the agent_run_shell_command tool."""
 
-    @agent.tool(strict=False)
+    @agent.tool
     def agent_run_shell_command(
         context: RunContext, command: str = "", cwd: str = None, timeout: int = 60
     ) -> ShellCommandOutput:
@@ -619,8 +560,8 @@ def register_agent_run_shell_command(agent):
                 - success (bool): True if command executed successfully (exit code 0)
                 - command (str | None): The executed command string
                 - error (str | None): Error message if execution failed
-                - stdout (str | None): Standard output from the command (last 1000 lines)
-                - stderr (str | None): Standard error from the command (last 1000 lines)
+                - stdout (str | None): Standard output from the command (last 256 lines)
+                - stderr (str | None): Standard error from the command (last 256 lines)
                 - exit_code (int | None): Process exit code
                 - execution_time (float | None): Total execution time in seconds
                 - timeout (bool | None): True if command was terminated due to timeout
@@ -651,7 +592,7 @@ def register_agent_run_shell_command(agent):
 def register_agent_share_your_reasoning(agent):
     """Register only the agent_share_your_reasoning tool."""
 
-    @agent.tool(strict=False)
+    @agent.tool
     def agent_share_your_reasoning(
         context: RunContext, reasoning: str = "", next_steps: str | None = None
     ) -> ReasoningOutput:
