@@ -149,7 +149,7 @@ class CodePuppyTUI(App):
     def on_mount(self) -> None:
         """Initialize the application when mounted."""
         # Register this app instance for global access
-        from code_puppy.state_management import set_tui_app_instance
+        from code_puppy.tui_state import set_tui_app_instance
 
         set_tui_app_instance(self)
 
@@ -430,8 +430,10 @@ class CodePuppyTUI(App):
                 else:
                     # Only cancel the agent task if NO processes were killed
                     self._current_worker.cancel()
-                    state_management._message_history = prune_interrupted_tool_calls(
-                        state_management._message_history
+                    state_management.set_message_history(
+                        prune_interrupted_tool_calls(
+                            state_management.get_message_history()
+                        )
                     )
                     self.add_system_message("⚠️  Processing cancelled by user")
                     # Stop spinner and clear state only when agent is actually cancelled
